@@ -1,14 +1,8 @@
-/**
- * @file id_payload.h
- * 
- * @brief Interface of id_payload_t.
- * 
- */
-
 /*
  * Copyright (C) 2007 Tobias Brunner
  * Copyright (C) 2005-2006 Martin Willi
  * Copyright (C) 2005 Jan Hutter
+ *
  * Hochschule fuer Technik Rapperswil
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -20,6 +14,8 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
+ *
+ * $Id: id_payload.c 3589 2008-03-13 14:14:44Z martin $
  */
 
 #include <stddef.h>
@@ -49,7 +45,7 @@ struct private_id_payload_t {
 	/**
 	 * Next payload type.
 	 */
-	payload_type_t next_payload;
+	u_int8_t next_payload;
 
 	/**
 	 * Critical flag.
