@@ -1,10 +1,3 @@
-/**
- * @file id_payload.h
- * 
- * @brief Interface of id_payload_t.
- * 
- */
-
 /*
  * Copyright (C) 2007 Tobias Brunner
  * Copyright (C) 2005-2006 Martin Willi
@@ -20,8 +13,14 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
+ *
+ * $Id: id_payload.h 3589 2008-03-13 14:14:44Z martin $
  */
 
+/**
+ * @defgroup id_payload id_payload
+ * @{ @ingroup payloads
+ */
 
 #ifndef ID_PAYLOAD_H_
 #define ID_PAYLOAD_H_
@@ -34,8 +33,6 @@ typedef struct id_payload_t id_payload_t;
 
 /**
  * Length of a id payload without the data in bytes.
- * 
- * @ingroup payloads
  */
 #define ID_PAYLOAD_HEADER_LENGTH 8
 
@@ -43,12 +40,6 @@ typedef struct id_payload_t id_payload_t;
  * Object representing an IKEv2 ID payload.
  *
  * The ID payload format is described in RFC section 3.5.
- *
- * @b Constructors:
- * - id_payload_create_from_identification()
- * - id_payload_create()
- *
- * @ingroup payloads
  */
 struct id_payload_t {
 	/**
@@ -57,90 +48,77 @@ struct id_payload_t {
 	payload_t payload_interface;
 
 	/**
-	 * @brief Set the ID type.
+	 * Set the ID type.
 	 *
-	 * @param this 			calling id_payload_t object
 	 * @param type			Type of ID
 	 */
 	void (*set_id_type) (id_payload_t *this, id_type_t type);
 	
 	/**
-	 * @brief Get the ID type.
+	 * Get the ID type.
 	 *
-	 * @param this 			calling id_payload_t object
 	 * @return				type of the ID 
 	 */
 	id_type_t (*get_id_type) (id_payload_t *this);
 	
 	/**
-	 * @brief Set the ID data.
+	 * Set the ID data.
 	 * 
 	 * Data are getting cloned.
 	 *
-	 * @param this 			calling id_payload_t object
 	 * @param data			ID data as chunk_t
 	 */
 	void (*set_data) (id_payload_t *this, chunk_t data);
 	
 	/**
-	 * @brief Get the ID data.
+	 * Get the ID data.
 	 * 
 	 * Returned data are a copy of the internal one
 	 *
-	 * @param this 			calling id_payload_t object
 	 * @return				ID data as chunk_t
 	 */
 	chunk_t (*get_data_clone) (id_payload_t *this);
 	
 	/**
-	 * @brief Get the ID data.
+	 * Get the ID data.
 	 * 
 	 * Returned data are NOT copied.
 	 *
-	 * @param this 			calling id_payload_t object
 	 * @return				ID data as chunk_t
 	 */
 	chunk_t (*get_data) (id_payload_t *this);
 
 	/**
-	 * @brief Creates an identification object of this id payload.
+	 * Creates an identification object of this id payload.
 	 * 
 	 * Returned object has to get destroyed by the caller.
 	 *
-	 * @param this 			calling id_payload_t object
 	 * @return				identification_t object 
 	 */
 	identification_t *(*get_identification) (id_payload_t *this);
 	
 	/**
-	 * @brief Destroys an id_payload_t object.
-	 *
-	 * @param this 	id_payload_t object to destroy
+	 * Destroys an id_payload_t object.
 	 */
 	void (*destroy) (id_payload_t *this);
 };
 
 /**
- * @brief Creates an empty id_payload_t object.
+ * Creates an empty id_payload_t object.
  * 
  * @param payload_type	one of ID_INITIATOR, ID_RESPONDER
  * @return				id_payload_t object
- * 
- * @ingroup payloads
  */
 id_payload_t *id_payload_create(payload_type_t payload_type);
 
 /**
- * @brief Creates an id_payload_t from an existing identification_t object.
+ * Creates an id_payload_t from an existing identification_t object.
  * 
  * @param payload_type		one of ID_INITIATOR, ID_RESPONDER
  * @param identification	identification_t object
  * @return					id_payload_t object
- * 
- * @ingroup payloads
  */
-id_payload_t *id_payload_create_from_identification(payload_type_t payload_type, identification_t *identification);
+id_payload_t *id_payload_create_from_identification(payload_type_t payload_type,
+											identification_t *identification);
 
-
-
-#endif /* ID_PAYLOAD_H_ */
+#endif /* ID_PAYLOAD_H_ @} */
