@@ -1,10 +1,3 @@
-/**
- * @file job.h
- * 
- * @brief Interface job_t.
- * 
- */
-
 /*
  * Copyright (C) 2005-2006 Martin Willi
  * Copyright (C) 2005 Jan Hutter
@@ -19,6 +12,13 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
+ *
+ * $Id: job.h 3589 2008-03-13 14:14:44Z martin $
+ */
+
+/**
+ * @defgroup job job
+ * @{ @ingroup jobs
  */
 
 #ifndef JOB_H_
@@ -28,38 +28,27 @@ typedef struct job_t job_t;
 
 #include <library.h>
 
-
 /**
- * @brief Job-Interface as it is stored in the job queue.
- * 
- * @b Constructors:
- * - None, use specific implementation of the interface.
- * 
- * @ingroup jobs
+ * Job-Interface as it is stored in the job queue.
  */
 struct job_t {
 
 	/**
-	 * @brief Execute a job.
+	 * Execute a job.
 	 * 
 	 * The processing facility executes a job using this method. Jobs are
 	 * one-shot, they destroy themself after execution, so don't use a job
 	 * once it has been executed.
-	 *
-	 * @param this 				calling object
 	 */
 	void (*execute) (job_t *this);
 
 	/**
-	 * @brief Destroy a job.
+	 * Destroy a job.
 	 *
 	 * Is only called whenever a job was not executed (e.g. due daemon shutdown).
 	 * After execution, jobs destroy themself.
-	 * 
-	 * @param job_t calling object
 	 */
 	void (*destroy) (job_t *job);
 };
 
-#endif /* JOB_H_ */
-
+#endif /* JOB_H_ @} */
