@@ -1,10 +1,3 @@
-/**
- * @file task_manager.h
- * 
- * @brief Interface of task_manager_t.
- * 
- */
-
 /*
  * Copyright (C) 2006 Martin Willi
  * Hochschule fuer Technik Rapperswil
@@ -18,6 +11,13 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
+ *
+ * $Id: task_manager.h 3589 2008-03-13 14:14:44Z martin $
+ */
+
+/**
+ * @defgroup task_manager task_manager
+ * @{ @ingroup sa
  */
 
 #ifndef TASK_MANAGER_H_
@@ -32,42 +32,32 @@ typedef struct task_manager_t task_manager_t;
 
 /**
  * First retransmit timeout in milliseconds.
- *
- * @ingroup sa
  */
 #define RETRANSMIT_TIMEOUT 4000
 
 /**
  * Base which is raised to the power of the retransmission try.
- *
- * @ingroup sa
  */
 #define RETRANSMIT_BASE 1.8
 
 /**
  * Number of retransmits done before giving up.
- *
- * @ingroup sa
  */
 #define RETRANSMIT_TRIES 5
 
 /**
  * Interval for mobike routability checks in ms.
- *
- * @ingroup sa
  */
 #define ROUTEABILITY_CHECK_INTERVAL 2500
 
 /**
  * Number of routability checks before giving up
- * 
- * @ingroup sa
  */
 #define ROUTEABILITY_CHECK_TRIES 10
 
 
 /**
- * @brief The task manager, juggles task and handles message exchanges.
+ * The task manager, juggles task and handles message exchanges.
  *
  * On incoming requests, the task manager creates new tasks on demand and
  * juggles the request through all available tasks. Each task inspects the
@@ -97,18 +87,12 @@ typedef struct task_manager_t task_manager_t;
  
    @endberbatim
  * The peer is considered dead after 2min 45s when no reply comes in.
- *
- * @b Constructors:
- *  - task_manager_create()
- * 
- * @ingroup sa
  */
 struct task_manager_t {
 
 	/**
-	 * @brief Process an incoming message.
+	 * Process an incoming message.
 	 * 
-	 * @param this 			calling object
 	 * @param message		message to add payloads to
 	 * @return
 	 * 						- DESTROY_ME if IKE_SA must be closed
@@ -117,28 +101,24 @@ struct task_manager_t {
 	status_t (*process_message) (task_manager_t *this, message_t *message);
 
 	/**
-	 * @brief Initiate an exchange with the currently queued tasks.
-	 *
-	 * @param this			calling object
+	 * Initiate an exchange with the currently queued tasks.
 	 */
 	status_t (*initiate) (task_manager_t *this);
 
 	/**
-	 * @brief Queue a task in the manager.
+	 * Queue a task in the manager.
 	 *
-	 * @param this			calling object
 	 * @param task			task to queue
 	 */
 	void (*queue_task) (task_manager_t *this, task_t *task);
 
 	/**
-	 * @brief Retransmit a request if it hasn't been acknowledged yet.
+	 * Retransmit a request if it hasn't been acknowledged yet.
 	 *
 	 * A return value of INVALID_STATE means that the message was already
 	 * acknowledged and has not to be retransmitted. A return value of SUCCESS
 	 * means retransmission was required and the message has been resent.
 	 * 
-	 * @param this 			calling object
 	 * @param message_id	ID of the message to retransmit
 	 * @return
 	 * 						- INVALID_STATE if retransmission not required
@@ -147,52 +127,45 @@ struct task_manager_t {
 	status_t (*retransmit) (task_manager_t *this, u_int32_t message_id);
 
 	/**
-	 * @brief Migrate all tasks from other to this.
+	 * Migrate all tasks from other to this.
 	 *
 	 * To rekey or reestablish an IKE_SA completely, all queued or active
 	 * tasks should get migrated to the new IKE_SA.
 	 * 
-	 * @param this 			manager which gets all tasks
 	 * @param other			manager which gives away its tasks
 	 */
 	void (*adopt_tasks) (task_manager_t *this, task_manager_t *other);
 	
 	/**
-	 * @brief Reset message ID counters of the task manager.
+	 * Reset message ID counters of the task manager.
 	 *
 	 * The IKEv2 protocol requires to restart exchanges with message IDs
 	 * reset to zero (INVALID_KE_PAYLOAD, COOKIES, ...). The reset() method
 	 * resets the message IDs and resets all active tasks using the migrate()
 	 * method.
 	 * 
-	 * @param this 			calling object
 	 * @param other			manager which gives away its tasks
 	 */
 	void (*reset) (task_manager_t *this);
 	
 	/**
-	 * @brief Check if we are currently waiting for a reply.
+	 * Check if we are currently waiting for a reply.
 	 *
-	 * @param this			calling object
 	 * @return				TRUE if we are waiting, FALSE otherwise
 	 */
 	bool (*busy) (task_manager_t *this);
 	
 	/**
-	 * @brief Destroy the task_manager_t.
-	 *
-	 * @param this 			calling object
+	 * Destroy the task_manager_t.
 	 */
 	void (*destroy) (task_manager_t *this);
 };
 
 /**
- * @brief Create an instance of the task manager.
+ * Create an instance of the task manager.
  *
  * @param ike_sa		IKE_SA to manage.
- *
- * @ingroup sa
  */
 task_manager_t *task_manager_create(ike_sa_t *ike_sa);
 
-#endif /* TASK_MANAGER_H_ */
+#endif /* TASK_MANAGER_H_ @} */
