@@ -1,10 +1,3 @@
-/**
- * @file ike_dpd.h
- * 
- * @brief Interface ike_dpd_t.
- * 
- */
-
 /*
  * Copyright (C) 2007 Martin Willi
  * Hochschule fuer Technik Rapperswil
@@ -18,6 +11,13 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
+ *
+ * $Id: ike_dpd.h 3589 2008-03-13 14:14:44Z martin $
+ */
+
+/**
+ * @defgroup ike_dpd ike_dpd
+ * @{ @ingroup tasks
  */
 
 #ifndef IKE_DPD_H_
@@ -30,14 +30,9 @@ typedef struct ike_dpd_t ike_dpd_t;
 #include <sa/tasks/task.h>
 
 /**
- * @brief Task of type ike_dpd, detects dead peers.
+ * Task of type ike_dpd, detects dead peers.
  *
  * The DPD task actually does nothing, as a DPD has no associated payloads.
- *
- * @b Constructors:
- *  - ike_dpd_create()
- * 
- * @ingroup tasks
  */
 struct ike_dpd_t {
 
@@ -48,11 +43,11 @@ struct ike_dpd_t {
 };
 
 /**
- * @brief Create a new ike_dpd task.
+ * Create a new ike_dpd task.
  *
  * @param initiator		TRUE if thask is the original initator
  * @return				ike_dpd task to handle by the task_manager
  */
 ike_dpd_t *ike_dpd_create(bool initiator);
 
-#endif /* IKE_DPD_H_ */
+#endif /* IKE_DPD_H_ @} */
