@@ -1,10 +1,3 @@
-/**
- * @file task.c
- * 
- * @brief Enum values for task types
- * 
- */
-
 /*
  * Copyright (C) 2007 Tobias Brunner
  * Copyright (C) 2007 Martin Willi
@@ -19,6 +12,8 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
+ *
+ * $Id: task.c 3666 2008-03-26 18:40:19Z tobias $
  */
 
 #include "task.h"
@@ -29,15 +24,16 @@ ENUM(task_type_names, IKE_INIT, CHILD_REKEY,
 	"IKE_MOBIKE",
 	"IKE_AUTHENTICATE",
 	"IKE_AUTH_LIFETIME",
-	"IKE_CERT",
+	"IKE_CERT_PRE",
+	"IKE_CERT_POST",
 	"IKE_CONFIG",
 	"IKE_REKEY",
 	"IKE_REAUTH",
 	"IKE_DELETE",
 	"IKE_DPD",
-#ifdef P2P
-	"IKE_P2P",
-#endif /* P2P */
+#ifdef ME
+	"IKE_ME",
+#endif /* ME */
 	"CHILD_CREATE",
 	"CHILD_DELETE",
 	"CHILD_REKEY",
