@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: demux.c 3252 2007-10-06 21:24:50Z andreas $
+ * RCSID $Id: demux.c 3686 2008-03-28 11:48:14Z martin $
  */
 
 /* Ordering Constraints on Payloads
@@ -2167,7 +2167,7 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 
 	    /* Schedule for whatever timeout is specified */
 	    {
-		time_t delay;
+		time_t delay = UNDEFINED_TIME;
 		enum event_type kind = smc->timeout_event;
 		bool agreed_time = FALSE;
 		struct connection *c = st->st_connection;
