@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: kernel.c 3252 2007-10-06 21:24:50Z andreas $
+ * RCSID $Id: kernel.c 3846 2008-04-18 17:01:45Z andreas $
  */
 
 #include <stddef.h>
@@ -1827,30 +1827,30 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
 	static const struct esp_info esp_info[] = {
 	    { ESP_NULL, AUTH_ALGORITHM_HMAC_MD5,
 		0, HMAC_MD5_KEY_LEN,
-		SADB_EALG_NULL, SADB_AALG_MD5_HMAC },
+		SADB_EALG_NULL, SADB_AALG_MD5HMAC },
 	    { ESP_NULL, AUTH_ALGORITHM_HMAC_SHA1,
 		0, HMAC_SHA1_KEY_LEN,
-		SADB_EALG_NULL, SADB_AALG_SHA1_HMAC },
+		SADB_EALG_NULL, SADB_AALG_SHA1HMAC },
 
 	    { ESP_DES, AUTH_ALGORITHM_NONE,
 		DES_CBC_BLOCK_SIZE, 0,
-		SADB_EALG_DES_CBC, SADB_AALG_NONE },
+		SADB_EALG_DESCBC, SADB_AALG_NONE },
 	    { ESP_DES, AUTH_ALGORITHM_HMAC_MD5,
 		DES_CBC_BLOCK_SIZE, HMAC_MD5_KEY_LEN,
-		SADB_EALG_DES_CBC, SADB_AALG_MD5_HMAC },
+		SADB_EALG_DESCBC, SADB_AALG_MD5HMAC },
 	    { ESP_DES, AUTH_ALGORITHM_HMAC_SHA1,
 		DES_CBC_BLOCK_SIZE,
-		HMAC_SHA1_KEY_LEN, SADB_EALG_DES_CBC, SADB_AALG_SHA1_HMAC },
+		HMAC_SHA1_KEY_LEN, SADB_EALG_DESCBC, SADB_AALG_SHA1HMAC },
 
 	    { ESP_3DES, AUTH_ALGORITHM_NONE,
 		DES_CBC_BLOCK_SIZE * 3, 0,
-		SADB_EALG_3DES_CBC, SADB_AALG_NONE },
+		SADB_EALG_3DESCBC, SADB_AALG_NONE },
 	    { ESP_3DES, AUTH_ALGORITHM_HMAC_MD5,
 		DES_CBC_BLOCK_SIZE * 3, HMAC_MD5_KEY_LEN,
-		SADB_EALG_3DES_CBC, SADB_AALG_MD5_HMAC },
+		SADB_EALG_3DESCBC, SADB_AALG_MD5HMAC },
 	    { ESP_3DES, AUTH_ALGORITHM_HMAC_SHA1,
 		DES_CBC_BLOCK_SIZE * 3, HMAC_SHA1_KEY_LEN,
-		SADB_EALG_3DES_CBC, SADB_AALG_SHA1_HMAC },
+		SADB_EALG_3DESCBC, SADB_AALG_SHA1HMAC },
 	};
 
 	u_int8_t natt_type = 0;
@@ -1976,11 +1976,11 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
 	switch (st->st_ah.attrs.auth)
 	{
 	case AUTH_ALGORITHM_HMAC_MD5:
-	    authalg = SADB_AALG_MD5_HMAC;
+	    authalg = SADB_AALG_MD5HMAC;
 	    break;
 
 	case AUTH_ALGORITHM_HMAC_SHA1:
-	    authalg = SADB_AALG_SHA1_HMAC;
+	    authalg = SADB_AALG_SHA1HMAC;
 	    break;
 
 	default:
