@@ -1,4 +1,4 @@
-/* C code produced by gperf version 3.0.1 */
+/* C code produced by gperf version 3.0.3 */
 /* Command-line: /usr/bin/gperf -C -G -t  */
 /* Computed positions: -k'1-2,$' */
 
@@ -31,7 +31,6 @@ error "gperf generated tables don't work with this execution character set. Plea
 
 
 /* strongSwan keywords
- * Copyright (C) 2007 Tobias Brunner
  * Copyright (C) 2005 Andreas Steffen
  * Hochschule fuer Technik Rapperswil, Switzerland
  *
@@ -45,7 +44,7 @@ error "gperf generated tables don't work with this execution character set. Plea
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: keywords.txt 3267 2007-10-08 19:57:54Z andreas $
+ * RCSID $Id: keywords.txt 3928 2008-05-11 07:59:00Z andreas $
  */
 
 #include <string.h>
@@ -57,12 +56,12 @@ struct kw_entry {
     kw_token_t token;
 };
 
-#define TOTAL_KEYWORDS 98
+#define TOTAL_KEYWORDS 100
 #define MIN_WORD_LENGTH 3
 #define MAX_WORD_LENGTH 17
-#define MIN_HASH_VALUE 15
-#define MAX_HASH_VALUE 236
-/* maximum key range = 222, duplicates = 0 */
+#define MIN_HASH_VALUE 6
+#define MAX_HASH_VALUE 263
+/* maximum key range = 258, duplicates = 0 */
 
 #ifdef __GNUC__
 __inline
@@ -76,191 +75,194 @@ hash (str, len)
      register const char *str;
      register unsigned int len;
 {
-  static const unsigned char asso_values[] =
+  static const unsigned short asso_values[] =
     {
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237,  40,
-        5, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237,  90, 237,  25,
-       75,   5,  85,   0,  95,   0, 237,  55,   0,  45,
-        0,  70,  20, 237,  15,  70,  40,  20,   5, 237,
-        5,  65,   0, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237, 237, 237, 237, 237,
-      237, 237, 237, 237, 237, 237
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264,  10,
+        0, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264,  15, 264,   0,
+      100,   5,  90,  85,  60,   0, 264,  60,  10,  55,
+       80,  75,  15, 264,   0,  50,  35,   5,  25, 264,
+       10,  75,   0, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264, 264, 264, 264, 264,
+      264, 264, 264, 264, 264, 264
     };
   return len + asso_values[(unsigned char)str[1]] + asso_values[(unsigned char)str[0]] + asso_values[(unsigned char)str[len - 1]];
 }
 
 static const struct kw_entry wordlist[] =
   {
-    {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""},
     {""}, {""}, {""}, {""}, {""}, {""},
-    {"leftupdown",        KW_LEFTUPDOWN},
-    {""},
-    {"leftfirewall",      KW_LEFTFIREWALL},
-    {""}, {""}, {""},
-    {"leftsubnetwithin",  KW_LEFTSUBNETWITHIN},
-    {""}, {""}, {""},
-    {"virtual_private",   KW_VIRTUAL_PRIVATE},
-    {"rightupdown",       KW_RIGHTUPDOWN},
-    {""},
-    {"rightfirewall",     KW_RIGHTFIREWALL},
-    {"rekeyfuzz",         KW_REKEYFUZZ},
-    {"plutodebug",        KW_PLUTODEBUG},
-    {"rekeymargin",       KW_REKEYMARGIN},
-    {"rightsubnetwithin", KW_RIGHTSUBNETWITHIN},
-    {""},
-    {"leftnatip",         KW_LEFTNATIP},
-    {""},
-    {"leftnexthop",       KW_LEFTNEXTHOP},
-    {"leftsourceip",      KW_LEFTSOURCEIP},
-    {"p2p_mediation",     KW_P2P_MEDIATION},
-    {""}, {""}, {""}, {""}, {""}, {""},
-    {"rightnatip",        KW_RIGHTNATIP},
     {"crluri",            KW_CRLURI},
+    {"crluri2",           KW_CRLURI2},
+    {""}, {""}, {""}, {""}, {""}, {""},
+    {"rekeyfuzz",         KW_REKEYFUZZ},
+    {""}, {""},
+    {"crluri1",           KW_CRLURI},
+    {""}, {""}, {""},
+    {"certuribase",       KW_CERTURIBASE},
+    {"rightca",           KW_RIGHTCA},
+    {"rightfirewall",     KW_RIGHTFIREWALL},
+    {""},
+    {"rightnatip",        KW_RIGHTNATIP},
+    {"crlcheckinterval",  KW_CRLCHECKINTERVAL},
     {"rightnexthop",      KW_RIGHTNEXTHOP},
     {"rightsourceip",     KW_RIGHTSOURCEIP},
-    {"left",              KW_LEFT},
-    {""}, {""},
-    {"crluri2",           KW_CRLURI2},
-    {"leftcert",          KW_LEFTCERT,},
-    {""},
-    {"leftsubnet",        KW_LEFTSUBNET},
-    {"crlcheckinterval",  KW_CRLCHECKINTERVAL},
-    {"leftsendcert",      KW_LEFTSENDCERT},
-    {"leftprotoport",     KW_LEFTPROTOPORT},
-    {""},
+    {""}, {""}, {""}, {""}, {""}, {""}, {""},
+    {"leftca",            KW_LEFTCA},
+    {"leftfirewall",      KW_LEFTFIREWALL},
+    {"eap",               KW_EAP},
+    {"leftnatip",         KW_LEFTNATIP},
     {"right",             KW_RIGHT},
-    {""}, {""},
-    {"ike",               KW_IKE},
+    {"leftnexthop",       KW_LEFTNEXTHOP},
+    {"leftsourceip",      KW_LEFTSOURCEIP},
+    {""},
     {"rightcert",         KW_RIGHTCERT},
-    {"klipsdebug",        KW_KLIPSDEBUG},
+    {"virtual_private",   KW_VIRTUAL_PRIVATE},
     {"rightsubnet",       KW_RIGHTSUBNET},
     {""},
     {"rightsendcert",     KW_RIGHTSENDCERT},
     {"rightprotoport",    KW_RIGHTPROTOPORT},
-    {"plutostart",        KW_PLUTOSTART},
-    {"ikelifetime",       KW_IKELIFETIME},
-    {"keylife",           KW_KEYLIFE},
-    {""}, {""},
-    {"keep_alive",        KW_KEEP_ALIVE},
-    {"keyexchange",       KW_KEYEXCHANGE},
-    {""}, {""}, {""},
-    {"interfaces",        KW_INTERFACES},
-    {""},
-    {"leftallowany",      KW_LEFTALLOWANY},
-    {"leftrsasigkey",     KW_LEFTRSASIGKEY},
-    {""},
-    {"leftgroups",        KW_LEFTGROUPS},
-    {"leftid",            KW_LEFTID},
-    {"crluri1",           KW_CRLURI},
-    {"ldapbase",          KW_LDAPBASE},
-    {"lefthostaccess",    KW_LEFTHOSTACCESS},
-    {"rekey",             KW_REKEY},
-    {""},
-    {"pkcs11module",      KW_PKCS11MODULE},
-    {"rightallowany",     KW_RIGHTALLOWANY},
-    {"rightrsasigkey",    KW_RIGHTRSASIGKEY},
-    {"pkcs11keepstate",   KW_PKCS11KEEPSTATE},
-    {"rightgroups",       KW_RIGHTGROUPS},
-    {"rightid",           KW_RIGHTID},
-    {"esp",               KW_ESP},
-    {"uniqueids",         KW_UNIQUEIDS},
-    {"righthostaccess",   KW_RIGHTHOSTACCESS},
-    {"leftca",            KW_LEFTCA},
-    {"ocspuri",           KW_OCSPURI},
-    {"nat_traversal",     KW_NAT_TRAVERSAL},
-    {"dpdaction",         KW_DPDACTION},
-    {"p2p_mediated_by",   KW_P2P_MEDIATED_BY},
-    {"overridemtu",       KW_OVERRIDEMTU},
-    {""},
-    {"ocspuri2",          KW_OCSPURI2},
-    {""},
-    {"p2p_peerid",        KW_P2P_PEERID},
-    {""},
-    {"rightca",           KW_RIGHTCA},
-    {"prepluto",          KW_PREPLUTO},
-    {"type",              KW_TYPE},
-    {""},
-    {"eapdir",            KW_EAPDIR},
-    {"dumpdir",           KW_DUMPDIR},
-    {"eap",               KW_EAP},
-    {""}, {""},
-    {"reauth",            KW_REAUTH},
-    {""},
-    {"ldaphost",          KW_LDAPHOST},
-    {""},
-    {"modeconfig",        KW_MODECONFIG},
-    {"mobike",	           KW_MOBIKE},
-    {""},
-    {"fragicmp",          KW_FRAGICMP},
-    {""}, {""},
-    {"charondebug",       KW_CHARONDEBUG},
-    {""},
-    {"pfsgroup",          KW_PFSGROUP},
-    {""}, {""}, {""}, {""}, {""}, {""}, {""},
-    {"keyingtries",       KW_KEYINGTRIES},
-    {""},
-    {"ocspuri1",          KW_OCSPURI},
-    {""},
-    {"dpdtimeout",        KW_DPDTIMEOUT},
-    {""}, {""}, {""}, {""}, {""},
-    {"pkcs11proxy",       KW_PKCS11PROXY},
-    {""},
-    {"nocrsend",          KW_NOCRSEND},
-    {""}, {""}, {""}, {""}, {""},
-    {"pkcs11initargs",    KW_PKCS11INITARGS},
+    {""}, {""}, {""}, {""},
+    {"left",              KW_LEFT},
     {""},
     {"cacert",            KW_CACERT},
     {""},
+    {"leftcert",          KW_LEFTCERT,},
+    {""},
+    {"leftsubnet",        KW_LEFTSUBNET},
+    {"rightgroups",       KW_RIGHTGROUPS},
+    {"leftsendcert",      KW_LEFTSENDCERT},
+    {"leftprotoport",     KW_LEFTPROTOPORT},
+    {""},
+    {"righthostaccess",   KW_RIGHTHOSTACCESS},
+    {""}, {""},
+    {"ike",               KW_IKE},
+    {""},
+    {"plutostart",        KW_PLUTOSTART},
+    {"reauth",            KW_REAUTH},
+    {""},
+    {"esp",               KW_ESP},
+    {"cachecrls",         KW_CACHECRLS},
+    {"leftgroups",        KW_LEFTGROUPS},
+    {"ikelifetime",       KW_IKELIFETIME},
+    {"keylife",           KW_KEYLIFE},
     {"packetdefault",     KW_PACKETDEFAULT},
+    {"lefthostaccess",    KW_LEFTHOSTACCESS},
+    {"keep_alive",        KW_KEEP_ALIVE},
+    {"keyexchange",       KW_KEYEXCHANGE},
+    {"ocspuri",           KW_OCSPURI},
+    {"ocspuri2",          KW_OCSPURI2},
+    {"auth",              KW_AUTH},
+    {"rekey",             KW_REKEY},
+    {""}, {""},
+    {"rightallowany",     KW_RIGHTALLOWANY},
+    {"rightrsasigkey",    KW_RIGHTRSASIGKEY},
+    {"xauth",             KW_XAUTH},
+    {"rightupdown",       KW_RIGHTUPDOWN},
+    {"pkcs11module",      KW_PKCS11MODULE},
+    {"ocspuri1",          KW_OCSPURI},
+    {""},
+    {"pkcs11keepstate",   KW_PKCS11KEEPSTATE},
+    {"rekeymargin",       KW_REKEYMARGIN},
+    {"rightsubnetwithin", KW_RIGHTSUBNETWITHIN},
+    {"prepluto",          KW_PREPLUTO},
+    {"auto",              KW_AUTO},
+    {""},
+    {"authby",            KW_AUTHBY},
+    {"leftallowany",      KW_LEFTALLOWANY},
+    {"leftrsasigkey",     KW_LEFTRSASIGKEY},
     {"also",              KW_ALSO},
+    {"leftupdown",        KW_LEFTUPDOWN},
+    {"charonstart",       KW_CHARONSTART},
+    {"rightid",           KW_RIGHTID},
+    {""}, {""}, {""},
+    {"leftsubnetwithin",  KW_LEFTSUBNETWITHIN},
+    {"dumpdir",           KW_DUMPDIR},
+    {"fragicmp",          KW_FRAGICMP},
+    {""}, {""},
+    {"overridemtu",       KW_OVERRIDEMTU},
+    {"hidetos",           KW_HIDETOS},
+    {"nat_traversal",     KW_NAT_TRAVERSAL},
+    {"type",              KW_TYPE},
+    {"plutodebug",        KW_PLUTODEBUG},
+    {"leftid",            KW_LEFTID},
+    {""},
+    {"ldapbase",          KW_LDAPBASE},
+    {"plutostderrlog",    KW_PLUTOSTDERRLOG},
+    {""},
+    {"keyingtries",       KW_KEYINGTRIES},
+    {""},
+    {"pfsgroup",          KW_PFSGROUP},
+    {""}, {""}, {""}, {""},
+    {"compress",          KW_COMPRESS},
+    {""}, {""}, {""}, {""}, {""},
+    {"pkcs11initargs",    KW_PKCS11INITARGS},
+    {"interfaces",        KW_INTERFACES},
+    {"mobike",	           KW_MOBIKE},
+    {""}, {""},
+    {"uniqueids",         KW_UNIQUEIDS},
+    {""},
+    {"mediated_by",       KW_MEDIATED_BY},
+    {""}, {""},
+    {"mediation",         KW_MEDIATION},
+    {""}, {""}, {""},
+    {"ldaphost",          KW_LDAPHOST},
+    {""}, {""},
+    {"charondebug",       KW_CHARONDEBUG},
+    {""},
+    {"pfs",               KW_PFS},
+    {""},
+    {"dpdtimeout",        KW_DPDTIMEOUT},
+    {"pkcs11proxy",       KW_PKCS11PROXY},
+    {""}, {""}, {""},
+    {"klipsdebug",        KW_KLIPSDEBUG},
+    {""}, {""}, {""},
+    {"me_peerid",         KW_ME_PEERID},
+    {""}, {""}, {""}, {""},
+    {"postpluto",         KW_POSTPLUTO},
+    {"strictcrlpolicy",   KW_STRICTCRLPOLICY},
+    {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""},
+    {"force_keepalive",   KW_FORCE_KEEPALIVE},
+    {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""},
     {""}, {""}, {""},
     {"dpddelay",          KW_DPDDELAY},
-    {"postpluto",         KW_POSTPLUTO},
-    {""},
-    {"charonstart",       KW_CHARONSTART},
-    {"hidetos",           KW_HIDETOS},
-    {"compress",          KW_COMPRESS},
-    {""}, {""}, {""}, {""},
-    {"pfs",               KW_PFS},
-    {""}, {""},
-    {"authby",            KW_AUTHBY},
-    {""}, {""},
-    {"auto",              KW_AUTO},
     {""}, {""}, {""}, {""}, {""},
-    {"strictcrlpolicy",   KW_STRICTCRLPOLICY},
-    {""}, {""}, {""},
-    {"cachecrls",         KW_CACHECRLS},
-    {"xauth",             KW_XAUTH},
-    {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""},
-    {""}, {""}, {""}, {""},
-    {"auth",              KW_AUTH},
+    {"dpdaction",         KW_DPDACTION},
     {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""},
     {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""},
-    {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""},
-    {"forceencaps",       KW_FORCEENCAPS}
+    {""}, {""},
+    {"modeconfig",        KW_MODECONFIG},
+    {"forceencaps",       KW_FORCEENCAPS},
+    {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""},
+    {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""},
+    {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""},
+    {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""},
+    {"nocrsend",          KW_NOCRSEND}
   };
 
 #ifdef __GNUC__
 __inline
+#ifdef __GNUC_STDC_INLINE__
+__attribute__ ((__gnu_inline__))
+#endif
 #endif
 const struct kw_entry *
 in_word_set (str, len)
